@@ -9,23 +9,14 @@ from abc import ABCMeta
 import copy
 from collections import defaultdict
 import sys
+import operator
 import numpy as np
+import pandas as pd
 from pandas import Series as SecuritiesValues
+from PyFin.Utilities import to_dict
 from PyFin.Math.Accumulators.StatefulAccumulators import Shift
-from PyFin.Math.Accumulators.IAccumulators import CompoundedValueHolder
+from PyFin.Math.Accumulators.StatelessAccumulators import Latest
 from PyFin.Math.Accumulators.IAccumulators import Identity
-from PyFin.Math.Accumulators.IAccumulators import AddedValueHolder
-from PyFin.Math.Accumulators.IAccumulators import MinusedValueHolder
-from PyFin.Math.Accumulators.IAccumulators import MultipliedValueHolder
-from PyFin.Math.Accumulators.IAccumulators import DividedValueHolder
-from PyFin.Math.Accumulators.IAccumulators import LtOperatorValueHolder
-from PyFin.Math.Accumulators.IAccumulators import LeOperatorValueHolder
-from PyFin.Math.Accumulators.IAccumulators import GtOperatorValueHolder
-from PyFin.Math.Accumulators.IAccumulators import GeOperatorValueHolder
-from PyFin.Math.Accumulators.IAccumulators import EqOperatorValueHolder
-from PyFin.Math.Accumulators.IAccumulators import NeOperatorValueHolder
-from PyFin.Env.Settings import Settings
-from PyFin.Utilities import pyFinAssert
 
 if sys.version_info > (3, 0, 0):
     div_attr = "truediv"
@@ -37,29 +28,29 @@ class SecurityValueHolder(object):
     __metaclass__ = ABCMeta
 
     def __init__(self, dependency='x'):
-        self._symbolList = set()
         if isinstance(dependency, SecurityValueHolder):
             self._dependency = dependency._dependency
+            self._compHolder = copy.deepcopy(dependency)
+            self._window = self._compHolder._window
         else:
+            self._compHolder = None
             if not isinstance(dependency, str) and len(dependency) == 1:
                 self._dependency = [dependency[0].lower()]
+
             elif not isinstance(dependency, str) and len(dependency) >= 1:
                 self._dependency = [name.lower() for name in dependency]
             else:
                 self._dependency = [dependency.lower()]
-        self._window = 1
+            self._window = 0
         self._returnSize = 1
         self._holderTemplate = None
+        self.updated = False
+        self.cached = pd.Series()
+        self._innerHolders = {}
 
     @property
     def symbolList(self):
-        return list(self._symbolList)
-
-    @property
-    def dependency(self):
-        return {
-            symbol: self.fields for symbol in self.symbolList
-        }
+        return set(self._innerHolders.keys())
 
     @property
     def fields(self):
@@ -77,59 +68,68 @@ class SecurityValueHolder(object):
         return self._window
 
     def push(self, data):
-        names = set(data.keys())
-        for name in names:
-            if name in self.symbolList:
-                self.holders[name].push(data[name])
-            else:
-                self._symbolList.add(name)
-                self.holders[name] = copy.deepcopy(self._holderTemplate)
-                self.holders[name].push(data[name])
+        self.updated = False
+
+        if self._compHolder:
+            self._compHolder.push(data)
+            data = self._compHolder.value
+
+            for name in data.index:
+                try:
+                    self._innerHolders[name].push({'x': data[name]})
+                except KeyError:
+                    self._innerHolders[name] = copy.deepcopy(self._holderTemplate)
+                    self._innerHolders[name].push({'x':  data[name]})
+
+        else:
+            for name in data:
+                try:
+                    self._innerHolders[name].push(data[name])
+                except KeyError:
+                    self._innerHolders[name] = copy.deepcopy(self._holderTemplate)
+                    self._innerHolders[name].push(data[name])
 
     @property
     def value(self):
-        res = {}
-        for name in self.holders:
-            try:
-                res[name] = self.holders[name].value
-            except ArithmeticError:
-                res[name] = np.nan
-        return SecuritiesValues(res)
+        if self.updated:
+            return SecuritiesValues(self.cached)
+        else:
+            keys = self._innerHolders.keys()
+            values = []
+            for name in keys:
+                try:
+                    values.append(self._innerHolders[name].result())
+                except ArithmeticError:
+                    values.append(np.nan)
+            self.cached = dict(zip(keys, values))
+            self.updated = True
+            return SecuritiesValues(values, index=keys)
+
+    def value_by_name(self, name):
+        if self.updated:
+            return self.cached[name]
+        else:
+            return self._innerHolders[name].result()
 
     @property
     def holders(self):
         return self._innerHolders
 
     def isFullByName(self, name):
-        return self.holders[name].isFull
+        return self._innerHolders[name].isFull
 
     @property
     def isFull(self):
-        res = {}
-        for name in self.holders:
-            try:
-                res[name] = self.holders[name].isFull
-            except ArithmeticError:
-                res[name] = np.nan
-        return SecuritiesValues(res)
+        for name in self._innerHolders:
+            if not self._innerHolders[name].isFull:
+                return False
+        return True
 
-    def __getitem__(self, item):
-        try:
-            return self.holders[item].result()
-        except (TypeError, KeyError) as _:
-
-            if isinstance(item, tuple):
-                symbolList = set(i.lower() for i in item)
-                pyFinAssert(len(symbolList) == len(item), ValueError,
-                            "security name can't be duplicated")
-                res = SecuritiesValues(
-                    {s: self.holders[s].result() for s in symbolList}
-                )
-                return res
-            elif isinstance(item, SecurityValueHolder):
-                return FilteredSecurityValueHolder(self, item)
-            else:
-                raise TypeError("{0} is not a valid index".format(item))
+    def __getitem__(self, filter):
+        if isinstance(filter, SecurityValueHolder):
+            return FilteredSecurityValueHolder(self, filter)
+        else:
+            return self.value[filter]
 
     def __add__(self, right):
         return SecurityAddedValueHolder(self, right)
@@ -161,9 +161,6 @@ class SecurityValueHolder(object):
     def __rtruediv__(self, left):
         return SecurityDividedValueHolder(left, self)
 
-    def __rshift__(self, right):
-        return SecurityCompoundedValueHolder(self, right)
-
     def __lt__(self, right):
         return SecurityLtOperatorValueHolder(self, right)
 
@@ -182,9 +179,60 @@ class SecurityValueHolder(object):
     def __ne__(self, right):
         return SecurityNeOperatorValueHolder(self, right)
 
+    def __neg__(self):
+        return SecurityNegValueHolder(self)
+
     def shift(self, n):
         return SecurityShiftedValueHolder(self, n)
 
+    def transform(self, data, name=None, category_field=None):
+
+        for f in self._dependency:
+            if f not in data:
+                raise ValueError('({0}) dependency is not in input data'.format(f))
+
+        data = data.sort_index()
+        dummy_category = False
+        if not category_field:
+            category_field = 'dummy'
+            data[category_field] = 1
+            dummy_category = True
+            total_index = list(range(len(data)))
+        else:
+            total_index = data.index
+
+        if not name:
+            name = 'transformed'
+
+        total_category = data[category_field].tolist()
+        matrix_values = data.as_matrix()
+        columns = data.columns.tolist()
+        split_category, split_values = to_dict(total_index, total_category, matrix_values, columns)
+
+        output_values = np.zeros((len(data), 1))
+
+        start_count = 0
+        if not dummy_category:
+            for j, dict_data in enumerate(split_values):
+                self.push(dict_data)
+                end_count = start_count + len(dict_data)
+                output_values[start_count:end_count, 0] = self.value[split_category[j]]
+                start_count = end_count
+        else:
+            for j, dict_data in enumerate(split_values):
+                self.push(dict_data)
+                output_values[j, 0] = self.value_by_name(split_category[j][0])
+
+        df = pd.DataFrame(output_values, index=total_category, columns=[name])
+
+        if dummy_category:
+            df.index = data.index
+            return df
+        else:
+            df[category_field] = df.index
+            df.index = data.index
+            return df
+
 
 class RankedSecurityValueHolder(SecurityValueHolder):
 
@@ -196,13 +244,32 @@ class RankedSecurityValueHolder(SecurityValueHolder):
             raise ValueError("Currently only value holder input is allowed for rank holder.")
         self._window = self._inner.window
         self._returnSize = self._inner.valueSize
-        self._dependency = self._inner.dependency
-        self._symbolList = self._inner._symbolList
+        self._dependency = copy.deepcopy(self._inner._dependency)
+        self.updated = False
+        self.cached = pd.Series()
+
+    @property
+    def symbolList(self):
+        return self._inner.symbolList
 
     @property
     def value(self):
-        raw_values = self._inner.value
-        return raw_values.rank()
+        if self.updated:
+            return self.cached
+        else:
+            raw_values = self._inner.value
+            self.cached = raw_values.rank(ascending=False)
+            self.updated = True
+            return self.cached
+
+    def value_by_name(self, name):
+        if self.updated:
+            return self.cached[name]
+        else:
+            raw_values = self._inner.value
+            self.cached = raw_values.rank(ascending=False)
+            self.updated = True
+            return self.cached[name]
 
     @property
     def holders(self):
@@ -210,6 +277,7 @@ class RankedSecurityValueHolder(SecurityValueHolder):
 
     def push(self, data):
         self._inner.push(data)
+        self.updated = False
 
 
 class FilteredSecurityValueHolder(SecurityValueHolder):
@@ -222,9 +290,12 @@ class FilteredSecurityValueHolder(SecurityValueHolder):
             self._computer._dependency,
             self._filter._dependency
         )
-        self._symbolList = self._computer._symbolList
-        self._updated = False
-        self._cachedFlag = None
+        self.updated = False
+        self.cached = pd.Series()
+
+    @property
+    def symbolList(self):
+        return self._computer.symbolList
 
     @property
     def holders(self):
@@ -232,165 +303,227 @@ class FilteredSecurityValueHolder(SecurityValueHolder):
 
     @property
     def value(self):
-        res = {}
-        for name in self.symbolList:
-            if self._filter[name]:
-                try:
-                    res[name] = self.holders[name].value
-                except ArithmeticError:
-                    res[name] = np.nan
-        return SecuritiesValues(res)
+        if self.updated:
+            return self.cached
+        else:
+            filter_value = self._filter.value
+            self.cached = self._computer.value[filter_value != 0]
+            self.updated = True
+            return self.cached
 
-    def __getitem__(self, item):
-        try:
-            if self._filter[item]:
-                return self.holders[item].result()
+    def value_by_name(self, name):
+        if self.updated:
+            return self.cached[name]
+        else:
+            filter_value = self._filter.value_by_name(name)
+            if filter_value:
+                return self._computer.value_by_name(name)
             else:
                 return np.nan
-        except KeyError:
-
-            if isinstance(item, SecurityValueHolder):
-                return FilteredSecurityValueHolder(self, item)
-            else:
-                raise TypeError("{0} is not a valid id".format(item))
 
     def push(self, data):
         self._computer.push(data)
         self._filter.push(data)
+        self.updated = False
 
 
 class IdentitySecurityValueHolder(SecurityValueHolder):
-    def __init__(self, value, n=1):
+    def __init__(self, value):
         self._value = value
-        self._symbolList = []
-        self._window = 1
-        self._returnSize = n
+        self._window = 0
+        self._returnSize = 1
         self._dependency = []
-        self._holderTemplate = Identity(value, n)
+        self._innerHolders = {}
+        self._holderTemplate = Identity(value)
+        self.updated = False
+        self.cached = pd.Series()
+
+    def push(self, data):
+        for name in data:
+            try:
+                self._innerHolders[name].push(data)
+            except KeyError:
+                self._innerHolders[name] = copy.deepcopy(self._holderTemplate)
+                self._innerHolders[name].push(data)
+
+        self.updated = False
+
+
+class SecurityUnitoryValueHolder(SecurityValueHolder):
+
+    def __init__(self, right, op):
+        self._right = copy.deepcopy(right)
+
+        self._window = self._right.window
+        self._dependency = copy.deepcopy(self._right._dependency)
+        self._returnSize = self._right.valueSize
+        self._op = op
+        self.updated = False
+        self.cached = pd.Series()
+
+    @property
+    def symbolList(self):
+        return self._right.symbolList
+
+    def push(self, data):
+        self._right.push(data)
+        self.updated = False
+
+    @property
+    def value(self):
+        if self.updated:
+            return self.cached
+        else:
+            self.cached = self._op(self._right.value)
+            self.updated = True
+            return self.cached
+
+    def value_by_name(self, name):
+        if self.updated:
+            return self.cached[name]
+        else:
+            return self._op(self._right.value_by_name(name))
+
+
+class SecurityNegValueHolder(SecurityUnitoryValueHolder):
+    def __init__(self, right):
+        super(SecurityNegValueHolder, self).__init__(
+            right, operator.neg)
+
+
+class SecurityLatestValueHolder(SecurityValueHolder):
+    def __init__(self, dependency='x'):
+        super(SecurityLatestValueHolder, self).__init__(dependency)
+        if self._compHolder:
+            self._holderTemplate = Latest(dependency='x')
+            self._innerHolders = {
+                name: copy.deepcopy(self._holderTemplate) for name in self._compHolder.symbolList
+                }
+        else:
+            self._holderTemplate = Latest(dependency=self._dependency)
 
 
 class SecurityCombinedValueHolder(SecurityValueHolder):
-    def __init__(self, left, right, HolderType):
+    def __init__(self, left, right, op):
         if isinstance(left, SecurityValueHolder):
             self._left = copy.deepcopy(left)
             if isinstance(right, SecurityValueHolder):
                 self._right = copy.deepcopy(right)
-                self._symbolList = set(self._left.symbolList).union(
-                    set(right.symbolList))
+            elif isinstance(right, str):
+                self._right = SecurityLatestValueHolder(right)
             else:
                 self._right = IdentitySecurityValueHolder(right)
-                self._symbolList = set(self._left.symbolList)
+        elif isinstance(left, str):
+            self._left = SecurityLatestValueHolder(left)
+            self._right = copy.deepcopy(right)
         else:
             self._left = IdentitySecurityValueHolder(left)
             self._right = copy.deepcopy(right)
-            self._symbolList = set(right.symbolList)
 
         self._window = max(self._left.window, self._right.window)
         self._dependency = _merge2set(
             self._left._dependency, self._right._dependency)
         self._returnSize = self._left.valueSize
+        self._op = op
+        self.updated = False
+        self.cached = pd.Series()
 
-        self._holderTemplate = HolderType(self._left._holderTemplate, self._right._holderTemplate)
+    @property
+    def symbolList(self):
+        return self._left.symbolList.union(self._right.symbolList)
 
-        self._innerHolders = {
-            name: copy.deepcopy(self._holderTemplate) for name in self._left.symbolList
-        }
+    def push(self, data):
+        self._left.push(data)
+        self._right.push(data)
+        self.updated = False
+
+    @property
+    def value(self):
+        if self.updated:
+            return self.cached
+        else:
+            self.cached = self._op(self._left.value, self._right.value)
+            self.updated = True
+            return self.cached
+
+    def value_by_name(self, name):
+        if self.updated:
+            return self.cached[name]
+        else:
+            return self._op(self._left.value_by_name(name), self._right.value_by_name(name))
 
 
 class SecurityAddedValueHolder(SecurityCombinedValueHolder):
     def __init__(self, left, right):
         super(SecurityAddedValueHolder, self).__init__(
-            left, right, AddedValueHolder)
+            left, right, operator.add)
 
 
 class SecuritySubbedValueHolder(SecurityCombinedValueHolder):
     def __init__(self, left, right):
         super(SecuritySubbedValueHolder, self).__init__(
-            left, right, MinusedValueHolder)
+            left, right, operator.sub)
 
 
 class SecurityMultipliedValueHolder(SecurityCombinedValueHolder):
     def __init__(self, left, right):
         super(SecurityMultipliedValueHolder, self).__init__(
-            left, right, MultipliedValueHolder)
+            left, right, operator.mul)
 
 
 class SecurityDividedValueHolder(SecurityCombinedValueHolder):
     def __init__(self, left, right):
         super(SecurityDividedValueHolder, self).__init__(
-            left, right, DividedValueHolder)
+            left, right, getattr(operator, div_attr))
 
 
 class SecurityLtOperatorValueHolder(SecurityCombinedValueHolder):
     def __init__(self, left, right):
         super(SecurityLtOperatorValueHolder, self).__init__(
-            left, right, LtOperatorValueHolder)
+            left, right, operator.lt)
 
 
 class SecurityLeOperatorValueHolder(SecurityCombinedValueHolder):
     def __init__(self, left, right):
         super(SecurityLeOperatorValueHolder, self).__init__(
-            left, right, LeOperatorValueHolder)
+            left, right, operator.le)
 
 
 class SecurityGtOperatorValueHolder(SecurityCombinedValueHolder):
     def __init__(self, left, right):
         super(SecurityGtOperatorValueHolder, self).__init__(
-            left, right, GtOperatorValueHolder)
+            left, right, operator.gt)
 
 
 class SecurityGeOperatorValueHolder(SecurityCombinedValueHolder):
     def __init__(self, left, right):
         super(SecurityGeOperatorValueHolder, self).__init__(
-            left, right, GeOperatorValueHolder)
+            left, right, operator.ge)
 
 
 class SecurityEqOperatorValueHolder(SecurityCombinedValueHolder):
     def __init__(self, left, right):
         super(SecurityEqOperatorValueHolder, self).__init__(
-            left, right, EqOperatorValueHolder)
+            left, right, operator.eq)
 
 
 class SecurityNeOperatorValueHolder(SecurityCombinedValueHolder):
     def __init__(self, left, right):
         super(SecurityNeOperatorValueHolder, self).__init__(
-            left, right, NeOperatorValueHolder)
+            left, right, operator.ne)
 
 
 class SecurityShiftedValueHolder(SecurityValueHolder):
 
     def __init__(self, right, n):
+        super(SecurityShiftedValueHolder, self).__init__(right)
         self._returnSize = right.valueSize
-        self._symbolList = set(right.symbolList)
         self._window = right.window + n
         self._dependency = copy.deepcopy(right._dependency)
-        self._holderTemplate = Shift(right._holderTemplate, n)
+        self._holderTemplate = Shift(Latest('x'), n)
 
         self._innerHolders = {
-            name: copy.deepcopy(self._holderTemplate) for name in self._symbolList
-        }
-
-
-class SecurityCompoundedValueHolder(SecurityValueHolder):
-    def __init__(self, left, right):
-        self._returnSize = right.valueSize
-        self._symbolList = set(left.symbolList)
-        self._window = left.window + right.window - 1
-        self._dependency = left.dependency
-        if not isinstance(right.fields, str):
-            pyFinAssert(left.valueSize == len(right.fields), ValueError, "left value size {0} is "
-                        "different from right dependency {1}"
-                        .format(left.valueSize, right.fields))
-        else:
-            pyFinAssert(left.valueSize == 1, ValueError, "left value size {0} is different from right dependency 1"
-                        .format(left.valueSize))
-
-        self._right = copy.deepcopy(right._holderTemplate)
-        self._left = copy.deepcopy(left._holderTemplate)
-
-        self._holderTemplate = CompoundedValueHolder(self._left, self._right)
-        self._innerHolders = {
-            name: copy.deepcopy(self._holderTemplate) for name in self._symbolList
+            name: copy.deepcopy(self._holderTemplate) for name in self._compHolder.symbolList
         }
 
 
