@@ -7,7 +7,6 @@ Created on 2015-7-16
 
 import math
 import bisect
-from collections import deque
 import numpy as np
 from copy import deepcopy
 from PyFin.Math.Accumulators.IAccumulators import Accumulator
@@ -15,11 +14,11 @@ from PyFin.Math.Accumulators.StatelessAccumulators import Latest
 from PyFin.Math.Accumulators.StatelessAccumulators import Positive
 from PyFin.Math.Accumulators.StatelessAccumulators import Negative
 from PyFin.Math.Accumulators.StatelessAccumulators import XAverage
-from PyFin.Math.Accumulators.StatelessAccumulators import StatelessAccumulator
+from PyFin.Math.Accumulators.StatelessAccumulators import StatelessSingleValueAccumulator
 from PyFin.Math.Accumulators.IAccumulators import Pow
-from PyFin.Math.Accumulators.IAccumulators import Identity
 from PyFin.Utilities import pyFinAssert
 from PyFin.Utilities import isClose
+from PyFin.Math.Accumulators.impl import Deque
 
 
 def _checkParameterList(dependency):
@@ -38,27 +37,18 @@ class StatefulValueHolder(Accumulator):
         self._returnSize = 1
         self._window = window
         self._containerSize = window
-        self._con = deque()
+        self._deque = Deque(window)
 
     @property
     def size(self):
-        return len(self._con)
+        return self._deque.size()
+
+    @property
+    def isFull(self):
+        return self._deque.isFull()
 
     def _dumpOneValue(self, value):
-
-        if self._isFull:
-            self._con.append(value)
-            popout = self._con.popleft()
-        else:
-            try:
-                popout = np.array([np.nan] * len(value))
-            except TypeError:
-                popout = np.nan
-
-            self._con.append(value)
-            if len(self._con) == self._containerSize:
-                self._isFull = 1
-        return popout
+        return self._deque.dump(value)
 
 
 class Shift(StatefulValueHolder):
@@ -73,7 +63,7 @@ class Shift(StatefulValueHolder):
 
     def push(self, data):
         self._valueHolder.push(data)
-        self._popout = super(Shift, self)._dumpOneValue(self._valueHolder.result())
+        self._popout = self._dumpOneValue(self._valueHolder.result())
 
     def result(self):
         try:
@@ -87,6 +77,17 @@ class SingleValuedValueHolder(StatefulValueHolder):
         super(SingleValuedValueHolder, self).__init__(window, dependency)
         _checkParameterList(dependency)
 
+    def _push(self, data):
+        if not self._isValueHolderContained:
+            try:
+                value = data[self._dependency]
+            except KeyError:
+                value = np.nan
+        else:
+            self._dependency.push(data)
+            value = self._dependency.result()
+        return value
+
 
 class SortedValueHolder(SingleValuedValueHolder):
     def __init__(self, window, dependency='x'):
@@ -94,10 +95,10 @@ class SortedValueHolder(SingleValuedValueHolder):
         self._sortedArray = []
 
     def push(self, data):
-        value = super(SortedValueHolder, self).push(data)
-        if np.isnan(value):
+        value = self._push(data)
+        if math.isnan(value):
             return np.nan
-        if self._isFull:
+        if self._deque.isFull():
             popout = self._dumpOneValue(value)
             delPos = bisect.bisect_left(self._sortedArray, popout)
             del self._sortedArray[delPos]
@@ -135,11 +136,11 @@ class MovingSum(SingleValuedValueHolder):
         self._runningSum = 0.0
 
     def push(self, data):
-        value = super(MovingSum, self).push(data)
-        if np.isnan(value):
+        value = self._push(data)
+        if math.isnan(value):
             return np.nan
         popout = self._dumpOneValue(value)
-        if not np.isnan(popout):
+        if not math.isnan(popout):
             self._runningSum = self._runningSum - popout + value
         else:
             self._runningSum = self._runningSum + value
@@ -154,14 +155,14 @@ class MovingAverage(SingleValuedValueHolder):
         self._runningSum = 0.0
 
     def push(self, data):
-        value = super(MovingAverage, self).push(data)
-        if np.isnan(value):
+        value = self._push(data)
+        if math.isnan(value):
             return np.nan
         popout = self._dumpOneValue(value)
-        if not np.isnan(popout):
-            self._runningSum = self._runningSum - popout + value
+        if not math.isnan(popout):
+            self._runningSum += value - popout
         else:
-            self._runningSum = self._runningSum + value
+            self._runningSum += value
 
     def result(self):
         try:
@@ -177,8 +178,8 @@ class MovingPositiveAverage(SingleValuedValueHolder):
         self._runningPositiveCount = 0
 
     def push(self, data):
-        value = super(MovingPositiveAverage, self).push(data)
-        if np.isnan(value):
+        value = self._push(data)
+        if math.isnan(value):
             return np.nan
         popout = self._dumpOneValue(value)
         if value > 0.0:
@@ -255,8 +256,8 @@ class MovingNegativeAverage(SingleValuedValueHolder):
         self._runningNegativeCount = 0
 
     def push(self, data):
-        value = super(MovingNegativeAverage, self).push(data)
-        if np.isnan(value):
+        value = self._push(data)
+        if math.isnan(value):
             return np.nan
         popout = self._dumpOneValue(value)
         if value < 0.0:
@@ -284,19 +285,19 @@ class MovingVariance(SingleValuedValueHolder):
             pyFinAssert(window >= 2, ValueError, "sampling variance can't be calculated with window size < 2")
 
     def push(self, data):
-        value = super(MovingVariance, self).push(data)
-        if np.isnan(value):
+        value = self._push(data)
+        if math.isnan(value):
             return np.nan
         popout = self._dumpOneValue(value)
-        if not np.isnan(popout):
-            self._runningSum = self._runningSum - popout + value
-            self._runningSumSquare = self._runningSumSquare - popout * popout + value * value
+        if not math.isnan(popout):
+            self._runningSum += value - popout
+            self._runningSumSquare += value * value - popout * popout
         else:
-            self._runningSum = self._runningSum + value
-            self._runningSumSquare = self._runningSumSquare + value * value
+            self._runningSum += value
+            self._runningSumSquare += value * value
 
     def result(self):
-        length = self.size
+        length = self._deque.size()
 
         if length == 0:
             return np.nan
@@ -321,8 +322,8 @@ class MovingNegativeVariance(SingleValuedValueHolder):
         self._isPop = isPopulation
 
     def push(self, data):
-        value = super(MovingNegativeVariance, self).push(data)
-        if np.isnan(value):
+        value = self._push(data)
+        if math.isnan(value):
             return np.nan
         popout = self._dumpOneValue(value)
         if value < 0:
@@ -357,8 +358,8 @@ class MovingCountedPositive(SingleValuedValueHolder):
         self._counts = 0
 
     def push(self, data):
-        value = super(MovingCountedPositive, self).push(data)
-        if np.isnan(value):
+        value = self._push(data)
+        if math.isnan(value):
             return np.nan
         popout = self._dumpOneValue(value)
 
@@ -377,8 +378,8 @@ class MovingCountedNegative(SingleValuedValueHolder):
         self._counts = 0
 
     def push(self, data):
-        value = super(MovingCountedNegative, self).push(data)
-        if np.isnan(value):
+        value = self._push(data)
+        if math.isnan(value):
             return np.nan
         popout = self._dumpOneValue(value)
 
@@ -399,7 +400,7 @@ class MovingHistoricalWindow(StatefulValueHolder):
     def push(self, data):
         value = super(MovingHistoricalWindow, self).push(data)
         try:
-            if np.isnan(value):
+            if math.isnan(value):
                 return np.nan
         except TypeError:
             if not value:
@@ -411,7 +412,7 @@ class MovingHistoricalWindow(StatefulValueHolder):
         if item >= length:
             raise ValueError("index {0} is out of the bound of the historical current length {1}".format(item, length))
 
-        return self._con[length - 1 - item]
+        return self._deque[length - 1 - item]
 
     def result(self):
         return [self.__getitem__(i) for i in range(self.size)]
@@ -429,10 +430,10 @@ class MovingCorrelation(StatefulValueHolder):
 
     def push(self, data):
         value = super(MovingCorrelation, self).push(data)
-        if np.any(np.isnan(value)):
+        if math.isnan(value[0]) or math.isnan(value[1]):
             return np.nan
         popout = self._dumpOneValue(value)
-        if not np.isnan(popout[0]):
+        if not math.isnan(popout[0]):
             headLeft = popout[0]
             headRight = popout[1]
 
@@ -453,7 +454,6 @@ class MovingCorrelation(StatefulValueHolder):
     def result(self):
         n = self.size
         if n >= 2:
-            n = self.size
             nominator = n * self._runningSumCrossSquare - self._runningSumLeft * self._runningSumRight
             denominator = (n * self._runningSumSquareLeft - self._runningSumLeft * self._runningSumLeft) \
                           * (n * self._runningSumSquareRight - self._runningSumRight * self._runningSumRight)
@@ -477,7 +477,7 @@ class MovingCorrelationMatrix(StatefulValueHolder):
 
     def push(self, data):
         values = super(MovingCorrelationMatrix, self).push(data)
-        if np.any(np.isnan(values)):
+        if math.isnan(sum(values)):
             return np.nan
         if self._isFirst:
             self._runningSum = np.zeros((1, len(values)))
@@ -495,8 +495,8 @@ class MovingCorrelationMatrix(StatefulValueHolder):
             self._runningSumCrossSquare += reshapeValues * reshapeValues.T
 
     def result(self):
-        if len(self._con) >= 2:
-            n = self.size
+        n = self.size
+        if n >= 2:
             nominator = n * self._runningSumCrossSquare - self._runningSum * self._runningSum.T
             denominator = n * np.diag(self._runningSumCrossSquare) - self._runningSum * self._runningSum
             denominator = np.sqrt(denominator * denominator.T)
@@ -508,16 +508,18 @@ class MovingCorrelationMatrix(StatefulValueHolder):
 class MovingProduct(SingleValuedValueHolder):
     def __init__(self, window, dependency='x'):
         super(MovingProduct, self).__init__(window, dependency)
-        self._runningProduct = np.nan
+        self._runningProduct = 1.0
+
     def push(self, data):
-        value = super(MovingProduct, self).push(data)
-        if np.isnan(value):
+        value = self._push(data)
+        if math.isnan(value):
             return np.nan
-        self._dumpOneValue(value)
-        if all(self._con):
-            self._runningProduct = np.product(self._con)
+        popout = self._dumpOneValue(value)
+        if not math.isnan(popout):
+            self._runningProduct *= value / popout
         else:
-            self._runningProduct = 0
+            self._runningProduct *= value
+
     def result(self):
         return self._runningProduct
 
@@ -528,12 +530,12 @@ class MovingCenterMoment(SingleValuedValueHolder):
         self._order = order
 
     def push(self, data):
-        value = super(MovingCenterMoment, self).push(data)
+        value = self._push(data)
         self._dumpOneValue(value)
-        if np.isnan(value):
+        if math.isnan(value):
             return np.nan
         else:
-            self._runningMoment = np.mean(np.power(np.abs(np.array(self._con) - np.mean(self._con)), self._order))
+            self._runningMoment = np.mean(np.power(np.abs(self._deque.as_array() - np.mean(self._deque.as_array())), self._order))
 
     def result(self):
         return self._runningMoment
@@ -541,7 +543,7 @@ class MovingCenterMoment(SingleValuedValueHolder):
 
 class MovingSkewness(SingleValuedValueHolder):
     def __init__(self, window, dependency='x'):
-        super(MovingSkewness, self).__init__(window,dependency)
+        super(MovingSkewness, self).__init__(window, dependency)
         self._runningStd3 = Pow(MovingVariance(window, dependency, isPopulation=True), 1.5)
         self._runningMoment3 = MovingCenterMoment(window, 3, dependency)
         self._runningSkewness = self._runningMoment3 / self._runningStd3
@@ -559,25 +561,27 @@ class MovingMaxPos(SortedValueHolder):
         self._runningTsMaxPos = np.nan
 
     def push(self, data):
-        super(MovingMaxPos,self).push(data)
+        super(MovingMaxPos, self).push(data)
         self._max = self._sortedArray[-1]
-        self._runningTsMaxPos = self._con.index(self._max)
 
     def result(self):
+        tmpList = self._deque.as_list()
+        self._runningTsMaxPos = tmpList.index(self._max)
         return self._runningTsMaxPos
 
 
 class MovingMinPos(SortedValueHolder):
     def __init__(self, window, dependency='x'):
-        super(MovingMinPos,self).__init__(window, dependency)
+        super(MovingMinPos, self).__init__(window, dependency)
         self._runningTsMinPos = np.nan
 
     def push(self, data):
-        super(MovingMinPos,self).push(data)
+        super(MovingMinPos, self).push(data)
         self._min = self._sortedArray[0]
-        self._runningTsMinPos = self._con.index(self._min)
 
     def result(self):
+        tmpList = self._deque.as_list()
+        self._runningTsMinPos = tmpList.index(self._min)
         return self._runningTsMinPos
 
 
@@ -601,19 +605,19 @@ class MovingRSV(SingleValuedValueHolder):
         self._cached_value = None
 
     def push(self, data):
-
-        value = super(MovingRSV, self).push(data)
-        if np.isnan(value):
+        value = self._push(data)
+        if math.isnan(value):
             return np.nan
         else:
             self._dumpOneValue(value)
             self._cached_value = value
 
     def result(self):
-        return (self._cached_value - min(self._con)) / (max(self._con) - min(self._con))
+        con = self._deque.as_list()
+        return (self._cached_value - min(con)) / (max(con) - min(con))
 
 
-class MACD(StatelessAccumulator):
+class MACD(StatelessSingleValueAccumulator):
     def __init__(self, short, long, dependency='x', method=XAverage):
         super(MACD, self).__init__(dependency)
         self._short_average = method(window=short, dependency=dependency)
@@ -634,15 +638,12 @@ class MovingRank(SortedValueHolder):
         super(MovingRank, self).__init__(window, dependency)
         self._runningRank = []
 
-    def push(self, data):
-        super(MovingRank, self).push(data)
-
     def result(self):
-        self._runningRank = [bisect.bisect_left(self._sortedArray, x) for x in self._con]
+        self._runningRank = [bisect.bisect_left(self._sortedArray, x) for x in self._deque.as_list()]
         return self._runningRank
 
 
-#runningJ can be more than 1 or less than 0.
+# runningJ can be more than 1 or less than 0.
 class MovingKDJ(SingleValuedValueHolder):
     def __init__(self, window, k=3, d=3, dependency='x'):
         super(MovingKDJ, self).__init__(window, dependency)
@@ -653,11 +654,11 @@ class MovingKDJ(SingleValuedValueHolder):
     def push(self, data):
         value = self._runningRsv.push(data)
         rsv = self._runningRsv.value
-        if len(self._con) == 0:
+        if self.size == 0:
             self._dumpOneValue(value)
             self._runningJ = np.nan
         else:
-            if len(self._con) == 1:
+            if self.size == 1:
                 self._dumpOneValue(value)
                 self._runningK = (0.5 * (self._k - 1) + rsv) / self._k
                 self._runningD = (0.5 * (self._d - 1) + self._runningK) / self._d
@@ -675,14 +676,15 @@ class MovingAroon(SingleValuedValueHolder):
         super(MovingAroon, self).__init__(window, dependency)
 
     def push(self, data):
-        value = super(MovingAroon, self).push(data)
-        if np.isnan(value):
+        value = self._push(data)
+        if math.isnan(value):
             return np.nan
         else:
             self._dumpOneValue(value)
 
     def result(self):
-        self._runningAroonOsc = (self._con.index(np.max(self._con)) - self._con.index(np.min(self._con))) / self.window
+        tmpList = self._deque.as_list()
+        self._runningAroonOsc = (tmpList.index(np.max(tmpList)) - tmpList.index(np.min(tmpList))) / self.window
         return self._runningAroonOsc
 
 
@@ -691,13 +693,13 @@ class MovingBias(SingleValuedValueHolder):
         super(MovingBias, self).__init__(window, dependency)
         self._runningMa = np.nan
 
-    def push(self,data):
-        value = super(MovingBias, self).push(data)
-        if np.isnan(value):
+    def push(self, data):
+        value = self._push(data)
+        if math.isnan(value):
             return np.nan
         else:
             self._dumpOneValue(value)
-            self._runningBias = value / np.mean(self._con) - 1
+            self._runningBias = value / np.mean(self._deque.as_array()) - 1
 
     def result(self):
         return self._runningBias
@@ -709,13 +711,14 @@ class MovingLevel(SingleValuedValueHolder):
         self._runningLevel = 1
 
     def push(self, data):
-        value = super(MovingLevel, self).push(data)
-        if np.isnan(value):
+        value = self._push(data)
+        if math.isnan(value):
             return np.nan
         else:
             self._dumpOneValue(value)
-            if len(self._con) > 1:
-                self._runningLevel = self._con[-1] / self._con[0]
+            if self.size > 1:
+                con = self._deque.as_list()
+                self._runningLevel = con[-1] / con[0]
 
     def result(self):
         return self._runningLevel
@@ -726,26 +729,34 @@ class MovingAutoCorrelation(SingleValuedValueHolder):
         super(MovingAutoCorrelation, self).__init__(window, dependency)
         self._lags = lags
         if window <= lags:
-            raise ValueError ("lags should be less than window however\n"
+            raise ValueError("lags should be less than window however\n"
                              "window is: {0} while lags is: {1}".format(window, lags))
 
     def push(self, data):
-        value = super(MovingAutoCorrelation, self).push(data)
-        if np.isnan(value):
+        value = self._push(data)
+        if math.isnan(value):
             return np.nan
         else:
             self._dumpOneValue(value)
 
     def result(self):
-        tmp_list = list(self._con)
-        if len(self._con) < self.window:
+        tmp_list = self._deque.as_list()
+        if len(tmp_list) < self.window:
             return np.nan
         else:
             try:
                 self._runningVecForward = tmp_list[0:self._window - self._lags]
                 self._runningVecBackward = tmp_list[-self._window + self._lags - 1:-1]
                 self._runningAutoCorrMatrix = np.cov(self._runningVecBackward, self._runningVecForward) / \
-                                        (np.std(self._runningVecBackward) * np.std(self._runningVecForward))
+                                              (np.std(self._runningVecBackward) * np.std(self._runningVecForward))
             except ZeroDivisionError:
                 return np.nan
-            return self._runningAutoCorrMatrix[0, 1]
\ No newline at end of file
+            return self._runningAutoCorrMatrix[0, 1]
+
+
+if __name__ == '__main__':
+
+    s = Shift(MovingAverage(2, 'x'), 1)
+    s.push({'x': 2})
+    s.push({'x': 3})
+    print(s.result())
\ No newline at end of file
