@@ -16,13 +16,34 @@ from PyFin.Analysis.TechnicalAnalysis import SecurityMovingCountedPositive
 from PyFin.Analysis.TechnicalAnalysis import SecurityMovingPositiveAverage
 from PyFin.Analysis.TechnicalAnalysis import SecurityMovingRSI
 from PyFin.Analysis.TechnicalAnalysis import SecurityMovingHistoricalWindow
-from PyFin.Analysis.TechnicalAnalysis import SecurityLatestValueHolder
+from PyFin.Analysis.TechnicalAnalysis import SecuritySignValueHolder
 from PyFin.Analysis.TechnicalAnalysis import SecurityXAverageValueHolder
 from PyFin.Analysis.TechnicalAnalysis import SecurityMACDValueHolder
 from PyFin.Analysis.TechnicalAnalysis import SecuritySqrtValueHolder
 from PyFin.Analysis.TechnicalAnalysis import SecurityDiffValueHolder
 from PyFin.Analysis.TechnicalAnalysis import SecuritySimpleReturnValueHolder
 from PyFin.Analysis.TechnicalAnalysis import SecurityLogReturnValueHolder
+from PyFin.Analysis.TechnicalAnalysis import SecurityExpValueHolder
+from PyFin.Analysis.TechnicalAnalysis import SecurityLogValueHolder
+from PyFin.Analysis.TechnicalAnalysis import SecurityPowValueHolder
+from PyFin.Analysis.TechnicalAnalysis import SecurityAbsValueHolder
+from PyFin.Analysis.TechnicalAnalysis import SecurityAcosValueHolder
+from PyFin.Analysis.TechnicalAnalysis import SecurityAcoshValueHolder
+from PyFin.Analysis.TechnicalAnalysis import SecurityAsinValueHolder
+from PyFin.Analysis.TechnicalAnalysis import SecurityAsinhValueHolder
+from PyFin.Analysis.SecurityValueHolders import SecurityShiftedValueHolder
+from PyFin.Analysis.SecurityValueHolders import SecurityLatestValueHolder
+
+
+from PyFin.Analysis.SecurityValueHolders import RankedSecurityValueHolder
+
+
+def CSRank(dependency):
+    return RankedSecurityValueHolder(dependency)
+
+
+def SIGN(dependency='x'):
+    return SecuritySignValueHolder(dependency)
 
 
 def EMA(window, dependency='x'):
@@ -89,6 +110,42 @@ def RETURNLog(dependency='x'):
     return SecurityLogReturnValueHolder(dependency)
 
 
+def EXP(dependency):
+    return SecurityExpValueHolder(dependency)
+
+
+def LOG(dependency):
+    return SecurityLogValueHolder(dependency)
+
+
+def POW(dependency):
+    return SecurityPowValueHolder(dependency)
+
+
+def ABS(dependency):
+    return SecurityAbsValueHolder(dependency)
+
+
+def ACOS(dependency):
+    return SecurityAcosValueHolder(dependency)
+
+
+def ACOSH(dependency):
+    return SecurityAcoshValueHolder(dependency)
+
+
+def ASIN(dependency):
+    return SecurityAsinValueHolder(dependency)
+
+
+def ASINH(dependency):
+    return SecurityAsinhValueHolder(dependency)
+
+
+def SHIFT(dependency, n):
+    return SecurityShiftedValueHolder(dependency, n)
+
+
 HIGH = functools.partial(LAST, 'high')
 LOW = functools.partial(LAST, 'low')
 OPEN = functools.partial(LAST, 'open')
