@@ -16,6 +16,8 @@ from PyFin.api.DateUtilities import advanceDateByCalendar
 from PyFin.api.DateUtilities import nthWeekDay
 from PyFin.api.DateUtilities import makeSchedule
 
+from PyFin.Analysis import transform
+from PyFin.api.Analysis import SIGN
 from PyFin.api.Analysis import EMA
 from PyFin.api.Analysis import MACD
 from PyFin.api.Analysis import RSI
@@ -35,6 +37,16 @@ from PyFin.api.Analysis import SQRT
 from PyFin.api.Analysis import DIFF
 from PyFin.api.Analysis import RETURNSimple
 from PyFin.api.Analysis import RETURNLog
+from PyFin.api.Analysis import CSRank
+from PyFin.api.Analysis import EXP
+from PyFin.api.Analysis import LOG
+from PyFin.api.Analysis import POW
+from PyFin.api.Analysis import ABS
+from PyFin.api.Analysis import ACOS
+from PyFin.api.Analysis import ACOSH
+from PyFin.api.Analysis import ASIN
+from PyFin.api.Analysis import ASINH
+from PyFin.api.Analysis import SHIFT
 
 
 __all__ = ["datesList",
@@ -47,6 +59,8 @@ __all__ = ["datesList",
            "advanceDateByCalendar",
            "nthWeekDay",
            "makeSchedule",
+           "transform",
+           "SIGN",
            "EMA",
            "MACD",
            "RSI",
@@ -65,4 +79,14 @@ __all__ = ["datesList",
            "SQRT",
            "DIFF",
            "RETURNSimple",
-           "RETURNLog"]
+           "RETURNLog",
+           "CSRank",
+           "EXP",
+           "LOG",
+           "POW",
+           "ABS",
+           "ACOS",
+           "ACOSH",
+           "ASIN",
+           "ASINH",
+           "SHIFT"]
