@@ -7,8 +7,10 @@ Created on 2015-8-11
 
 from PyFin.tests.Analysis.DataProviders import TestDataProviders
 from PyFin.tests.Analysis.testSecurityValueHolders import TestSecurityValueHolders
+from PyFin.tests.Analysis.testTransformer import TestTransformer
 from PyFin.tests.Analysis import TechnicalAnalysis
 
 __all__ = ['TestDataProviders',
            'TestSecurityValueHolders',
+           'TestTransformer',
            'TestTechnicalAnalysis']
