@@ -6,16 +6,17 @@ Created on 2015-8-12
 """
 
 import unittest
-from collections import deque
 import numpy as np
+import pandas as pd
 from PyFin.Enums import Factors
 from PyFin.Analysis.SecurityValueHolders import SecuritiesValues
 from PyFin.Analysis.SecurityValueHolders import dependencyCalculator
 from PyFin.Analysis.SecurityValueHolders import RankedSecurityValueHolder
 from PyFin.Analysis.SecurityValueHolders import FilteredSecurityValueHolder
-from PyFin.Analysis.TechnicalAnalysis import SecurityLatestValueHolder
+from PyFin.Analysis.SecurityValueHolders import SecurityLatestValueHolder
 from PyFin.Analysis.TechnicalAnalysis import SecurityMovingAverage
 from PyFin.Analysis.TechnicalAnalysis import SecurityMovingMax
+from PyFin.Analysis.TechnicalAnalysis import SecurityMovingMinimum
 from PyFin.Analysis.TechnicalAnalysis import SecurityMovingSum
 
 
@@ -49,7 +50,7 @@ class TestSecurityValueHolders(unittest.TestCase):
             benchmark.push(data)
             rankHolder.push(data)
             benchmarkValues = benchmark.value
-            np.testing.assert_array_almost_equal(benchmarkValues.rank(), rankHolder.value)
+            np.testing.assert_array_almost_equal(benchmarkValues.rank(ascending=False), rankHolder.value)
 
     def testFilteredSecurityValueHolder(self):
         benchmark = SecurityLatestValueHolder(dependency='close') > 0
@@ -62,10 +63,56 @@ class TestSecurityValueHolders(unittest.TestCase):
                             Factors.OPEN: self.datas['ibm'][Factors.OPEN][i]}}
             benchmark.push(data)
             filtered.push(data)
-            if np.isnan(filtered['aapl']):
-                self.assertTrue(filtered['aapl'])
-            if np.isnan(filtered['ibm']):
-                self.assertTrue(filtered['ibm'])
+            try:
+                _ = filtered['aapl']
+                self.assertTrue(benchmark['aapl'])
+            except KeyError:
+                self.assertFalse(benchmark['aapl'])
+
+            try:
+                _ = filtered['ibm']
+                self.assertTrue(benchmark['ibm'])
+            except KeyError:
+                self.assertFalse(benchmark['ibm'])
+
+    def testFilteredSecurityValueHolderWithGetItemMethod(self):
+        benchmark = SecurityLatestValueHolder(dependency='close') > 0
+        filtered = benchmark[benchmark]
+
+        for i in range(len(self.datas['aapl']['close'])):
+            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
+                             Factors.OPEN: self.datas['aapl'][Factors.OPEN][i]},
+                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
+                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i]}}
+            benchmark.push(data)
+            filtered.push(data)
+            try:
+                _ = filtered['aapl']
+                self.assertTrue(benchmark['aapl'])
+            except KeyError:
+                self.assertFalse(benchmark['aapl'])
+
+            try:
+                _ = filtered['ibm']
+                self.assertTrue(benchmark['ibm'])
+            except KeyError:
+                self.assertFalse(benchmark['ibm'])
+
+    def testSecurityValueHolderIsFull(self):
+        test = SecurityMovingMax(2, dependency='close')
+
+        data = {'aapl': {'close': 1.0},
+                'ibm': {'close': 2.0}}
+        test.push(data)
+        self.assertEqual(test.isFull, False)
+
+        data = {'aapl': {'close': 1.0}}
+        test.push(data)
+        self.assertEqual(test.isFull, False)
+
+        data = {'ibm': {'close': 13.0}}
+        test.push(data)
+        self.assertEqual(test.isFull, True)
 
     def testSecuritiesValuesComparison(self):
 
@@ -185,12 +232,7 @@ class TestSecurityValueHolders(unittest.TestCase):
 
         testValueHolder.push({'aapl': {'close': 1.0}, 'ibm': {'close': 2.0}})
 
-        dependency = {
-            name: pNames for name in symbolList
-            }
-
         self.assertEqual(set(testValueHolder.symbolList), set(symbolList))
-        self.assertEqual(testValueHolder.dependency, dependency)
         self.assertEqual(testValueHolder.valueSize, 1)
         self.assertEqual(testValueHolder.window, window)
 
@@ -199,29 +241,23 @@ class TestSecurityValueHolders(unittest.TestCase):
         pNames2 = ['open']
         test2 = SecurityMovingMax(window2, pNames2)
         binaryValueHolder = testValueHolder + test2
-        dependency2 = {
-            name: pNames + pNames2 for name in symbolList
-            }
 
         self.assertEqual(set(binaryValueHolder.symbolList), set(symbolList))
-        for name in dependency2:
-            self.assertEqual(set(binaryValueHolder.dependency[name]), set(dependency2[name]))
+
         self.assertEqual(binaryValueHolder.valueSize, 1)
         self.assertEqual(binaryValueHolder.window, max(window, window2))
 
         # test compounded operated value holder
         test3 = SecurityMovingMax(window2, testValueHolder)
         self.assertEqual(set(test3.symbolList), set(symbolList))
-        self.assertEqual(test3.dependency, dependency)
         self.assertEqual(test3.valueSize, 1)
-        self.assertEqual(test3.window, window + window2 - 1)
+        self.assertEqual(test3.window, window + window2)
 
         # test compounded twice
         test4 = SecurityMovingMax(window2, test3)
         self.assertEqual(set(test4.symbolList), set(symbolList))
-        self.assertEqual(test4.dependency, dependency)
         self.assertEqual(test4.valueSize, 1)
-        self.assertEqual(test4.window, window + 2 * window2 - 2)
+        self.assertEqual(test4.window, window + 2 * window2)
 
     def testDependencyCalculation(self):
         h1 = {'aapl': 'close', 'ibm': 'close'}
@@ -236,14 +272,6 @@ class TestSecurityValueHolders(unittest.TestCase):
         for name in expected:
             self.assertEqual(set(expected[name]), set(calculated[name]))
 
-    def testDependencyCalculationOnCompoundedValueHolder(self):
-        h = SecurityMovingMax(5, SecurityMovingAverage(10, 'close') + SecurityMovingAverage(20, 'open'))
-        h.push({'aapl': {'close': 5}})
-        expected = {'aapl': ['close', 'open']}
-        calculated = h.dependency
-        for name in expected:
-            self.assertEqual(set(calculated[name]), set(expected[name]))
-
     def testItemizedValueHolder(self):
         window = 10
         pNames = 'close'
@@ -251,19 +279,9 @@ class TestSecurityValueHolders(unittest.TestCase):
         test.push({'aapl': {'close': 10.0}, 'ibm': {'close': 15.0}, 'goog': {'close': 17.0}})
         test.push({'aapl': {'close': 12.0}, 'ibm': {'close': 10.0}, 'goog': {'close': 13.0}})
 
-        # single named value holder
-        test1 = test['ibm']
-        self.assertAlmostEqual(test1, 12.5, 15)
-
-        # multi-valued named value holder
-        test2 = test['ibm', 'goog']
-        expected = SecuritiesValues({'ibm': 12.5, 'goog':15.0})
-        for s in test2.index:
-            self.assertAlmostEqual(test2[s], expected[s])
-
-        # wrong type of item
-        with self.assertRaises(TypeError):
-            _ = test[1]
+        expected = {'ibm': 12.5, 'goog':15.0}
+        for s in expected:
+            self.assertAlmostEqual(test[s], expected[s])
 
     def testAddedSecurityValueHolders(self):
         window1 = 10
@@ -306,6 +324,25 @@ class TestSecurityValueHolders(unittest.TestCase):
             for name in expected.index:
                 self.assertAlmostEqual(expected[name], calculated[name], 12)
 
+    def testAddedSecurityValueHolderWithName(self):
+        window = 10
+        dependency = ['close']
+        mm = SecurityMovingSum(window, dependency)
+        combined = 'open' + mm
+
+        for i in range(len(self.datas['aapl']['close'])):
+            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
+                             Factors.OPEN: self.datas['aapl'][Factors.OPEN][i]},
+                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
+                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i]}}
+            mm.push(data)
+            combined.push(data)
+
+            base = mm.value
+            calculated = combined.value
+            for name in calculated.index:
+                self.assertAlmostEqual(base[name] + data[name]['open'], calculated[name], 12)
+
     def testRAddedSecurityValueHoldersWithScalar(self):
         window = 10
         dependency = ['close']
@@ -532,7 +569,7 @@ class TestSecurityValueHolders(unittest.TestCase):
         for name in expected.index:
             self.assertAlmostEqual(expected[name], calculated[name])
 
-        data3 = SecuritiesValues({'aapl': {'close': 3.0},
+        data3 = ({'aapl': {'close': 3.0},
                  'ibm': {'close': 4.0},
                  'goog': {'close': 5.0}})
 
@@ -549,32 +586,9 @@ class TestSecurityValueHolders(unittest.TestCase):
         with self.assertRaises(ValueError):
             _ = mm.shift(0)
 
-    def testCompoundedSecurityValueHolder(self):
-        ma = SecurityMovingAverage(2, 'close')
-        compounded = ma >> SecurityMovingMax(3)
-
-        container = {'aapl': deque(maxlen=3), 'ibm': deque(maxlen=3)}
-        expected = {'aapl': 0.0, 'ibm': 0.0}
-        for i in range(len(self.datas['aapl']['close'])):
-            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i]},
-                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i]}}
-            ma.push(data)
-            maRes = ma.value
-            for name in maRes.index:
-                container[name].append(maRes[name])
-                expected[name] = max(container[name])
-
-            compounded.push(data)
-            calculated = compounded.value
-            for name in calculated.index:
-                self.assertAlmostEqual(expected[name], calculated[name], 12, "for {0} at index {1}\n"
-                                                                             "expected:   {2}\n"
-                                                                             "calculated: {3}"
-                                       .format(name, i, expected[name], calculated[name]))
-
     def testLtSecurityValueHolder(self):
         filter = SecurityLatestValueHolder('close') < 10.0
-        ma = SecurityMovingAverage(10, 'close')[filter]
+        ma = FilteredSecurityValueHolder(SecurityMovingAverage(10, 'close'), filter)
 
         data = {'aapl': {'close': 15.},
                 'ibm': {'close': 8.},
@@ -598,7 +612,7 @@ class TestSecurityValueHolders(unittest.TestCase):
 
     def testLeSecurityValueHolder(self):
         filter = SecurityLatestValueHolder('close') <= 10.0
-        ma = SecurityMovingAverage(10, 'close')[filter]
+        ma = FilteredSecurityValueHolder(SecurityMovingAverage(10, 'close'), filter)
 
         data = {'aapl': {'close': 15.},
                 'ibm': {'close': 10.},
@@ -622,7 +636,7 @@ class TestSecurityValueHolders(unittest.TestCase):
 
     def testGtSecurityValueHolder(self):
         filter = SecurityLatestValueHolder('close') > 10.0
-        ma = SecurityMovingAverage(10, 'close')[filter]
+        ma = FilteredSecurityValueHolder(SecurityMovingAverage(10, 'close'), filter)
 
         data = {'aapl': {'close': 15.},
                 'ibm': {'close': 8.},
@@ -646,7 +660,7 @@ class TestSecurityValueHolders(unittest.TestCase):
 
     def testGeSecurityValueHolder(self):
         filter = SecurityMovingAverage(1, 'close') >= 10.0
-        ma = SecurityMovingAverage(10, 'close')[filter]
+        ma = FilteredSecurityValueHolder(SecurityMovingAverage(10, 'close'), filter)
 
         data = {'aapl': {'close': 15.},
                 'ibm': {'close': 10.},
@@ -667,3 +681,47 @@ class TestSecurityValueHolders(unittest.TestCase):
         calculated = ma.value
         for name in expected:
             self.assertAlmostEqual(expected[name], calculated[name], 15)
+
+    def testTransformWithCategory(self):
+        test_df = pd.DataFrame({'code': [1, 2, 3, 4, 1, 2, 3],
+                                'b': [4, 5, 6, 7, 6, 5, 4],
+                                'c': [9, 8, 7, 6, 5, 4, 3]},
+                               index=[1, 1, 1, 1, 2, 2, 2])
+
+        expression = SecurityMovingMax(20, 'b') + SecurityMovingMinimum(20, 'c')
+        calculated = expression.transform(test_df, name='new_factor', category_field='code')
+        expected = [13., 13., 13., 13., 11., 9, 9.]
+
+        np.testing.assert_array_almost_equal(calculated['new_factor'], expected)
+
+    def testTransformWithoutCategory(self):
+        test_df = pd.DataFrame({'b': [4, 5, 6, 7, 6, 5, 4],
+                                'c': [9, 8, 7, 6, 5, 4, 3]},
+                               index=[1, 2, 3, 4, 5, 6, 7])
+
+        expression = SecurityMovingMax(20, 'b') + SecurityMovingMinimum(20, 'c')
+        calculated = expression.transform(test_df, name='new_factor')
+        expected = [13., 13., 13., 13., 12., 11., 10.]
+        np.testing.assert_array_almost_equal(calculated['new_factor'], expected)
+
+    def testTransformWithDifferentGroup(self):
+        test_df = pd.DataFrame({'code': [1, 2, 3, 4, 5, 6, 7],
+                                'b': [4, 5, 6, 7, 6, 5, 4],
+                                'c': [9, 8, 7, 6, 5, 4, 3]},
+                               index=[1, 1, 1, 1, 2, 2, 2])
+
+        expression = SecurityMovingAverage(2, 'b')
+        calculated = expression.transform(test_df, name='new_factor', category_field='code')
+
+        expected = [4., 5., 6., 7., 6., 5., 4.]
+        np.testing.assert_array_almost_equal(calculated['new_factor'], expected)
+
+    def testTransformWithoutCategoryAndDuplicatedIndex(self):
+        sample_data = pd.DataFrame(
+            data={'code': [1, 2, 1, 2, 1, 2],
+                  'open': [2.0, 1.0, 1.5, 3.0, 2.4, 3.5],
+                  'close': [1.7, 1.6, 0.9, 3.8, 1.6, 2.1]},
+            index=[1, 1, 2, 2, 3, 3]
+        )
+
+
